#!/bin/sh

# Arguments.
RES_DIR="$1"
USER_ID="$2"
UPGRADING="$3"
DEVELOPING="$4"
APP_DIR="$1/../.."

# Utilities.
source "${RES_DIR}/utils.sh"

# Make sure we start with something clean.
DO_INCREMENT_PROGRESS_BAR="0"; export "DO_INCREMENT_PROGRESS_BAR";
"${RES_DIR}/uninstall.sh" "${RES_DIR}" "${USER_ID}" "${UPGRADING}" "${DEVELOPING}" ; EXIT_ON_ERROR 1 "uninstalling"
DO_INCREMENT_PROGRESS_BAR="1"

# Create the directories.
${MKDIR} -p "${HANDSOFF_DIR}" ; EXIT_ON_ERROR 2 "mkdir main"
${MKDIR} -p "${HANDSOFF_LOG_DIR}" ; EXIT_ON_ERROR 2 "mkdir log"
${CHMOD} 777 "${HANDSOFF_LOG_DIR}" ; EXIT_ON_ERROR 2 "chmod log"
${TOUCH} "${HANDSOFF_INSTALL_LOG}" ; EXIT_ON_ERROR 2 "touch install log"
${TOUCH} "${HANDSOFF_LOG_DIR}/${DAEMON_LOG_NAME}" ; EXIT_ON_ERROR 2 "touch daemon log"
${CHMOD} 644 "${HANDSOFF_LOG_DIR}/${DAEMON_LOG_NAME}" ; EXIT_ON_ERROR 2 "chmod daemon log"
${TOUCH} "${HANDSOFF_LOG_DIR}/${AGENT_LOG_NAME}" ; EXIT_ON_ERROR 2 "touch agent log"
${CHMOD} 666 "${HANDSOFF_LOG_DIR}/${AGENT_LOG_NAME}" ; EXIT_ON_ERROR 2 "chmod agent log"

# Log separator.
${ECHO} "----------------------------------------" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${ECHO} -n "install @ " >> ${HANDSOFF_INSTALL_LOG} 2>&1 
${DATE} >> ${HANDSOFF_INSTALL_LOG} 2>&1
${ECHO} "----------------------------------------" >> ${HANDSOFF_INSTALL_LOG} 2>&1

# Create alias and copy presets.
"${RES_DIR}/mkalias" "${APP_DIR}" "${APP_ALIAS}" ; EXIT_ON_ERROR 2 "mkalias"
${CP} -R "${PRESETS_SRC}" "${PRESETS_DST_DIR}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; EXIT_ON_ERROR 2 "cp presets"
INCREMENT_PROGRESS_BAR

# Install kext.
${CP} -R "${KEXT_SRC}" "${KEXT_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 3 "cp signed kext"
${CHOWN} -R root:wheel "${KEXT_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 3 "chown signed kext"
${TOUCH} "${KEXT_DST_DIR}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 3 "touch signed kext"
${SYNC} >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 3 "sync"
if [[ "$UPGRADING" == "0" || "$DEVELOPING" == "1" ]]; then
    ${KEXTLOAD} -v ${KEXT_DST} >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 4 "kextload signed"
fi

INCREMENT_PROGRESS_BAR

# Install daemon.
${CP} -R "${DAEMON_SRC}" "${DAEMON_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 5 "cp daemon"
${MKDIR} -p "${DAEMON_PLIST_DST_DIR}" ; EXIT_ON_ERROR 5 "mkdir daemon"
${CP} -R "${DAEMON_PLIST_SRC}" "${DAEMON_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 5 "cp daemon plist"
${LAUNCHCTL} load -w "${DAEMON_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 5 "launctl daemon"
INCREMENT_PROGRESS_BAR

# Install ui agent.
${CP} -R "${AGENT_SRC}" "${AGENT_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 6 "cp agent"
${MKDIR} -p "${AGENT_PLIST_DST_DIR}" ; EXIT_ON_ERROR 6 "mkdir agent"
${CP} -R "${AGENT_PLIST_SRC}" "${AGENT_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 6 "cp agent plist"
if [ "${OS_MINOR_VERSION}" -ge "15" ]; then
	${SUDO} -u\#${USER_ID} ${LAUNCHCTL} enable gui/${USER_ID}/${AGENT_ID} >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 6 "launchctl agent enable"
	${SUDO} -u\#${USER_ID} ${LAUNCHCTL} bootstrap gui/${USER_ID}/ "${AGENT_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 6 "launchctl agent bootstrap"
else
	${SUDO} -u\#${USER_ID} ${LAUNCHCTL} load -w "${AGENT_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1 ; UNINSTALL_AND_EXIT_ON_ERROR 6 "launchctl agent load"
fi
INCREMENT_PROGRESS_BAR

exit 0
