#!/bin/sh

# Arguments.
RES_DIR="$1"
USER_ID="$2"
UPGRADING="$3"
DEVELOPING="$4"

# Utilities.
source "${RES_DIR}/utils.sh"

# Create the directories.
${MKDIR} -p "${HANDSOFF_DIR}"
${MKDIR} -p "${HANDSOFF_LOG_DIR}"
${CHMOD} 777 "${HANDSOFF_LOG_DIR}"
${TOUCH} "${HANDSOFF_INSTALL_LOG}"

# Log separator.
${ECHO} "----------------------------------------" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${ECHO} -n "uninstall @ " >> ${HANDSOFF_INSTALL_LOG} 2>&1 
${DATE} >> ${HANDSOFF_INSTALL_LOG} 2>&1
${ECHO} "----------------------------------------" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${UNAME} -a >> ${HANDSOFF_INSTALL_LOG} 2>&1
${SW_VERS} >> ${HANDSOFF_INSTALL_LOG} 2>&1
${KEXTSTAT} | ${GREP} -v "com.apple" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${ECHO} "----------------------------------------" >> ${HANDSOFF_INSTALL_LOG} 2>&1

# Uninstall kext part 1.
if [ -e ${KEXT_DST} ]; then
	${RM} -rf "${KEXT_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
	${TOUCH} "${KEXT_DST_DIR}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
	if [ "${OS_MINOR_VERSION}" -ge "14" ]; then
		${KEXTCACHE} -i / 2>&1
	fi
	${SYNC} >> ${HANDSOFF_INSTALL_LOG} 2>&1
fi
INCREMENT_PROGRESS_BAR

# Uninstall alias, presets, repatch and restart DNS stuff.
${RM} -f "${APP_ALIAS}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${RM} -rf "${PRESETS_DIR}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${RM} -rf "${REPATCH_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${RM} -rf "${RESTARTDNS_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${RM} -rf "${KEXT_PATCH_INSTALL_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
INCREMENT_PROGRESS_BAR

# Uninstall DNS.
DNS_PLIST_DICT_TO_WRITE=`${DEFAULTS} read ${DNS_PLIST_DOMAIN} ${DNS_PLIST_KEY} 2>> ${HANDSOFF_INSTALL_LOG} | ${PERL} -pe 's|[{}]\s*||' | ${PERL} -pe 's|^\s*(.*?)\s*=\s*(.*?);\s*$|\1 \2\n|' | ${PERL} -pe "s|\"${DNS_PLIST_DICT_KEY}\" \"${DNS_PLIST_DICT_VALUE}\"||" | ${PERL} -pe "s|\"(${DNS_PLIST_DICT_KEY})\" \"${DNS_PLIST_DICT_VALUE}:(.*)\"|\"\1\" \"\2\"|" | ${PERL} -pe "s|\"(${DNS_PLIST_DICT_KEY})\" \"(.*:)${DNS_PLIST_DICT_VALUE}:(.*)\"|\"\1\" \"\2\3\"|" | ${PERL} -pe "s|\"(${DNS_PLIST_DICT_KEY})\" \"(.*):${DNS_PLIST_DICT_VALUE}\"|\"\1\" \"\2\"|" | ${PERL} -pe 's|(\s+)\n|\1 |'`
if [ "${DNS_PLIST_DICT_TO_WRITE}" == "" ]; then
	${DEFAULTS} delete ${DNS_PLIST_DOMAIN} ${DNS_PLIST_KEY} >> ${HANDSOFF_INSTALL_LOG} 2>&1
else
	${DEFAULTS} write ${DNS_PLIST_DOMAIN} ${DNS_PLIST_KEY} -dict ${DNS_PLIST_DICT_TO_WRITE} >> ${HANDSOFF_INSTALL_LOG} 2>&1
fi
${CHMOD} 644 "${DNS_PLIST}"
${RM} -rf "${DNS_LAUNCHER_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${RM} -rf "${DNS_LIB_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1

${LAUNCHCTL} unload "${DNS_PLIST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
${LAUNCHCTL} load "${DNS_PLIST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
INCREMENT_PROGRESS_BAR

# Uninstall ui agent.
if [ -e ${AGENT_PLIST_DST} ]; then
	if [ "${OS_MINOR_VERSION}" -ge "15" ]; then
		${SUDO} -u\#${USER_ID} ${LAUNCHCTL} bootout gui/${USER_ID}/ "${AGENT_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
		${SUDO} -u\#${USER_ID} ${LAUNCHCTL} disable gui/${USER_ID}/${AGENT_ID} >> ${HANDSOFF_INSTALL_LOG} 2>&1
	else
		${SUDO} -u\#${USER_ID} ${LAUNCHCTL} unload "${AGENT_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
	fi
fi
${RM} -rf "${AGENT_DST}" "${AGENT_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
INCREMENT_PROGRESS_BAR

# Uninstall daemon.
if [ -e ${DAEMON_PLIST_DST} ]; then
	${LAUNCHCTL} unload "${DAEMON_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
fi
${RM} -rf "${DAEMON_DST}" "${DAEMON_PLIST_DST}" >> ${HANDSOFF_INSTALL_LOG} 2>&1
INCREMENT_PROGRESS_BAR

# Uninstall kext part 2.
if [ "$DEVELOPING" == "1" ]; then
	${SYNC} >> ${HANDSOFF_INSTALL_LOG} 2>&1
	${KEXTUNLOAD} -v -b ${KEXT_IDENTIFIER} >> ${HANDSOFF_INSTALL_LOG} 2>&1
fi

exit 0
