#!/bin/sh

# Commands.
SYNC="/bin/sync"
KEXTLOAD="/sbin/kextload"
KEXTUNLOAD="/sbin/kextunload"
KEXTCACHE="/usr/sbin/kextcache"
TOUCH="/usr/bin/touch"
RM="/bin/rm"
CP="/bin/cp"
LAUNCHCTL="/bin/launchctl"
SUDO="/usr/bin/sudo"
DEFAULTS="/usr/bin/defaults"
MKDIR="/bin/mkdir"
PERL="/usr/bin/perl"
CHMOD="/bin/chmod"
ECHO="/bin/echo"
DATE="/bin/date"
CHOWN="/usr/sbin/chown"
UNAME="/usr/bin/uname"
SW_VERS="/usr/bin/sw_vers"
GREP="/usr/bin/grep"
KEXTSTAT="/usr/sbin/kextstat"
CUT="/usr/bin/cut"

# OS.
OS_MINOR_VERSION=`${SW_VERS} -productVersion | ${CUT} -d'.' -f2`

# Paths.
HANDSOFF_DIR="/Library/Handsoff"
HANDSOFF_LOG_DIR="${HANDSOFF_DIR}/log"
HANDSOFF_INSTALL_LOG="${HANDSOFF_LOG_DIR}/install.log"
APP_ALIAS="${HANDSOFF_DIR}/Hands Off!.app"

# Repatch.
REPATCH_NAME="repatch.sh"

REPATCH_DST_DIR="${HANDSOFF_DIR}"
REPATCH_DST="${HANDSOFF_DIR}/${REPATCH_NAME}"

# Restart DNS.
RESTARTDNS_NAME="restartdns.sh"

RESTARTDNS_DST_DIR="${HANDSOFF_DIR}"
RESTARTDNS_DST="${HANDSOFF_DIR}/${RESTARTDNS_NAME}"

# kext.
KEXT_NAME="HandsOff.kext"
KEXT_EXEC_NAME="HandsOff"
KEXT_PATCH_NAME="kext_patch"
KEXT_IDENTIFIER="com.metakine.handsoff.driver"

KEXT_SRC_DIR="${RES_DIR}/kext"
KEXT_SRC="${KEXT_SRC_DIR}/${KEXT_NAME}"

KEXT_DST_DIR="/Library/Extensions"
KEXT_DST="${KEXT_DST_DIR}/${KEXT_NAME}"
KEXT_PATCH_INSTALL_DST="${HANDSOFF_DIR}/${KEXT_PATCH_NAME}"

# daemon.
DAEMON_NAME="HandsOffDaemon"
DAEMON_PLIST_NAME="com.metakine.handsoff.daemon.plist"
DAEMON_LOG_NAME="handsoff_daemon.log"

DAEMON_SRC_DIR="${RES_DIR}/daemon"
DAEMON_SRC="${DAEMON_SRC_DIR}/${DAEMON_NAME}"
DAEMON_PLIST_SRC="${DAEMON_SRC_DIR}/${DAEMON_PLIST_NAME}"

DAEMON_DST_DIR="${HANDSOFF_DIR}"
DAEMON_DST="${DAEMON_DST_DIR}/${DAEMON_NAME}"
DAEMON_PLIST_DST_DIR="/Library/LaunchDaemons"
DAEMON_PLIST_DST="${DAEMON_PLIST_DST_DIR}/${DAEMON_PLIST_NAME}"

# presets
PRESETS_SRC="${DAEMON_SRC_DIR}/presets"
PRESETS_DST_DIR="${HANDSOFF_DIR}"
PRESETS_DIR="${HANDSOFF_DIR}/presets"

# agent.
AGENT_NAME="HandsOffAgent.app"
AGENT_ID="com.metakine.handsoff.agent"
AGENT_PLIST_NAME="${AGENT_ID}.plist"
AGENT_LOG_NAME="handsoff_agent.log"

AGENT_SRC_DIR="${RES_DIR}/uiagent"
AGENT_SRC="${AGENT_SRC_DIR}/${AGENT_NAME}"
AGENT_PLIST_SRC="${AGENT_SRC_DIR}/${AGENT_PLIST_NAME}"

AGENT_DST_DIR="${HANDSOFF_DIR}"
AGENT_DST="${AGENT_DST_DIR}/${AGENT_NAME}"
AGENT_PLIST_DST_DIR="/Library/LaunchAgents"
AGENT_PLIST_DST="${AGENT_PLIST_DST_DIR}/${AGENT_PLIST_NAME}"

# DNS.
DNS_LIB_NAME="dns_preload.dylib"
DNS_LAUNCHER_NAME="dnslauncher"

DNS_DST_DIR="${HANDSOFF_DIR}"
DNS_LIB_DST="${DNS_DST_DIR}/${DNS_LIB_NAME}"
DNS_LAUNCHER_DST="${DNS_DST_DIR}/${DNS_LAUNCHER_NAME}"

DNS_PLIST_DIR="/System/Library/LaunchDaemons"
DNS_PLIST="${DNS_PLIST_DIR}/com.apple.mDNSResponder.plist"
DNS_PLIST_DOMAIN="${DNS_PLIST_DIR}/com.apple.mDNSResponder"
DNS_PLIST_KEY="EnvironmentVariables"
DNS_PLIST_DICT_KEY="DYLD_INSERT_LIBRARIES"
DNS_PLIST_DICT_VALUE="${DNS_LIB_DST}"

if [ -z  "${DO_INCREMENT_PROGRESS_BAR}" ]; then
	DO_INCREMENT_PROGRESS_BAR="1";
fi

# Functions.
function INCREMENT_PROGRESS_BAR
{
	if [ "${DO_INCREMENT_PROGRESS_BAR}" -eq "1" ]; then
		${ECHO} -n "+"
	fi
}

function EXIT_ON_ERROR
{
	RETVALUE=$?
	if [ "$RETVALUE" -ne "0" ]; then
		${ECHO} "Error $RETVALUE processing $1 ($2)" >> ${HANDSOFF_INSTALL_LOG}
		exit $1
	fi
}

function UNINSTALL_AND_EXIT_ON_ERROR
{
	RETVALUE=$?
	if [ "$RETVALUE" -ne "0" ]; then
		${ECHO} "Error $RETVALUE processing $1 ($2), uninstalling..." >> ${HANDSOFF_INSTALL_LOG}
		"${RES_DIR}/uninstall.sh" "${RES_DIR}" "${USER_ID}" "${UPGRADING}" "${DEVELOPING}"
		exit $1
	fi
}
